/*****************************************************************************
* Copyright (c) 2005 Sena Technologies, Inc., All Rights Reserved
*
* This software contains proprietary and confidential information of Sena
* Technologies Inc.  By accepting transfer of this copy, Recipient agrees
* to retain this software in confidence, to prevent disclosure to others,
* and to make no use of this software other than that for which it was
* delivered.  This is an unpublished copyrighted work of Sena Technologies,
* Inc.  
*
* Sena Technologies, Inc. 210 Yangjae-dong Seocho-gu, Seoul, Korea
*
*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include "rc4.h"

#define BUF_SIZE 1024

void usage()
{
	printf("\nusage: rc4_client args\n\n");
	printf(" -host host              - use -connect instead\n");
	printf(" -port port              - use -connect instead\n");
	printf(" -connect host:port      - who to connect to\n");
	printf(" -key arg                - key string to use in rc4 encrytion/decryption\n");

	exit(-1);
}


int extract_host(char *host, int *port, char *str)
{
	char *p;
	
	p = strchr(str, ':');
	if (p == NULL) return -1;
	*p++ = '\0';
	sprintf(host, "%s", str);
	*port = atoi(p);
	return 0;
}	

int main(int argc, char **argv)
{
	int i;
	char host[64] = "";
	int port = -1;
	char key[64] = "";
	rc4_key enc_key, dec_key;
	fd_set readfds, writefds;
	char cbuf[BUF_SIZE], sbuf[BUF_SIZE];
	int cbuf_len=0, cbuf_off=0;
	int sbuf_len=0, sbuf_off=0;
	int s;
	struct hostent *hostinfo;
	struct sockaddr_in addr;
	int addr_len;
	int ret;
	int read_sock=1, read_tty=1, write_sock=0, write_tty=0;
	
	
	for (i=1; i<argc; i++) {
		if (!strcmp(argv[i], "-key")) {
			sprintf(key, "%s", argv[++i]);
		}
		else if (!strcmp(argv[i], "-host")) {
			sprintf(host, "%s", argv[++i]);
		}		
		else if (!strcmp(argv[i], "-port")) {
			port = atoi(argv[++i]);
		}		
		else if (!strcmp(argv[i], "-connect")) {
			if (extract_host(host, &port, argv[++i])) usage();
		}		
		else usage();
	}
	
	if (strlen(key) == 0 || strlen(host) == 0 || port <= 0) usage();

	/* connect to the host */
	
	hostinfo = gethostbyname(host);
	if (hostinfo == NULL) {
		printf("cannot resolve %s\n", host);
		exit(-1);
	}
	
	s = socket(AF_INET, SOCK_STREAM, 0);
	addr.sin_family = AF_INET;
	addr.sin_port = htons(port);
	addr.sin_addr = *(struct in_addr *) *hostinfo->h_addr_list;
	addr_len = sizeof(addr);
	
	ret = connect(s, (struct sockaddr *) &addr, addr_len);
	if (ret < 0) {
		printf("cannot connect to %s:%d\n", host, port);
		close(s);
		exit(-1);
	}
	
	printf("CONNECTED\n");
	
	/* initialize rc4 keys */
	
	prepare_key(key, strlen(key), &enc_key);
	prepare_key(key, strlen(key), &dec_key);


	for (;;) {
		FD_ZERO(&readfds);
		FD_ZERO(&writefds);
		
		if (read_sock) FD_SET(s, &readfds);
		if (read_tty) FD_SET(fileno(stdin), &readfds);
		if (write_sock) FD_SET(s, &writefds);
		if (write_tty) FD_SET(fileno(stdout), &writefds);
	
		ret = select(s+1, &readfds, &writefds, NULL, NULL);
		if (ret < 0) {
			printf("select() error\n");
			goto shutdown;
		}
		
		if (FD_ISSET(s, &readfds)) {
			ret = read(s, sbuf, sizeof(sbuf));
			if (ret>0) {
				rc4(sbuf, ret, &dec_key);
				sbuf_off = 0;
				sbuf_len = ret;
				read_sock = 0;
				write_tty = 1;
			}
		}
		if (FD_ISSET(fileno(stdin), &readfds)) {
			ret = read(fileno(stdin), cbuf, sizeof(cbuf));
			if (ret>0) {
				if (cbuf[0] == 'Q') {
					printf("DONE\n");
					goto shutdown;
				}
				rc4(cbuf, ret, &enc_key);
				cbuf_off = 0;
				cbuf_len = ret;
				read_tty = 0;
				write_sock = 1;
			}
		}
		if (FD_ISSET(s, &writefds)) {
			ret = write(s, cbuf+cbuf_off, cbuf_len);
			if (ret>0) {
				cbuf_off += ret;;
				cbuf_len -= ret;
				if (cbuf_len <=0) {
					read_tty = 1;
					write_sock = 0;
				}
			}
		}
		if (FD_ISSET(fileno(stdout), &writefds)) {
			ret = write(fileno(stdout), sbuf+sbuf_off, sbuf_len);
			if (ret>0) {
				sbuf_off += ret;;
				sbuf_len -= ret;
				if (sbuf_len <=0) {
					read_sock = 1;
					write_tty = 0;
				}
			}
		}

	}
shutdown:
	close(s);
	exit(0);	
	
}

